﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using Telerik.OpenAccess;
using Telerik.OpenAccess.SPI.dataobjects;
using System.Reflection;

namespace MockObjectScope
{
    public class PersistentClassRepository
    {
        private static Dictionary<Type, IList> persistentObjects = new Dictionary<Type, IList>();
        private static List<PersistenceCapable> tempListForInsert = new List<PersistenceCapable>();
        private static List<PersistenceCapable> tempListForRemove = new List<PersistenceCapable>();

        public static Dictionary<Type, IList> PersistentObjects
        {
            get { return PersistentClassRepository.persistentObjects; }
        }

        public static List<PersistenceCapable> TempListForInsert
        {
            get { return PersistentClassRepository.tempListForInsert; }
        }

        public static List<PersistenceCapable> TempListForRemove
        {
            get { return PersistentClassRepository.tempListForRemove; }
        }

        public static void InitializeRepository(List<Type> typesToBeUsed)
        {
            foreach (Type typ in typesToBeUsed)
            {
                persistentObjects[typ] = (IList)CreateGenericList(typ);
            }
        }

        public static void Add(object newObject)
        {
            if (!persistentObjects.Keys.Contains(newObject.GetType()))
            {
                persistentObjects[newObject.GetType()] = (IList)CreateGenericList(newObject.GetType());
                persistentObjects[newObject.GetType()].Add(newObject);
            }
            else
            {
                persistentObjects[newObject.GetType()].Add(newObject);
            }
        }

        private static object CreateGenericList(Type t)
        {
            Type genericList = typeof(List<>).MakeGenericType(t);
            ConstructorInfo constructor = genericList.GetConstructor(Type.EmptyTypes);
            return constructor.Invoke(null);
        }

        public static void Remove(object objectForRemoval)
        {
            if (!persistentObjects.Keys.Contains(objectForRemoval.GetType()))
            {
                throw new ArgumentException("You can not remove instances that are not related to the scope");
            }
            else if (persistentObjects[objectForRemoval.GetType()].Contains(objectForRemoval))
            {
                persistentObjects[objectForRemoval.GetType()].Remove(objectForRemoval);
            }
            else
            {
                throw new ArgumentException("You can not remove instances that are not related to the scope");
            }
        }

        public static void AddToTemporaryListForInsert(object newObject)
        {
            if (newObject.GetType().GetCustomAttributes(typeof(PersistentAttribute), true).Count() > 0)
            {
                foreach (PropertyInfo property in newObject.GetType().GetProperties())
                {
                    if (property.PropertyType.GetCustomAttributes(typeof(PersistentAttribute), true).Count() > 0)
                    {
                        object reference = property.GetValue(newObject, null);
                        if (reference != null)
                        {
                            AddToTemporaryListForInsert(property.GetValue(newObject, null));
                        }
                    }
                }
                tempListForInsert.Add((PersistenceCapable)newObject);
            }
            else
            {
                throw new ArgumentException("The instance is not persistence capable");
            }
        }

        public static void AddToTemporaryListForRemove(object objectForRemoval)
        {
            if (objectForRemoval.GetType().GetCustomAttributes(typeof(PersistentAttribute), true).Count() > 0)
            {
                tempListForRemove.Add((PersistenceCapable)objectForRemoval);
            }
            else
            {
                throw new ArgumentException("The instance is not persistence capable");
            }
        }
    }
}